@@ -3,14 +3,18 @@ use crate::csv::Row;
 use crate::find;
 use crate::sort;
 use crate::sort::SortOrder;
+use crate::theme::Theme;
 use crate::view;
 use crate::view::Header;
 use crate::wrap;
+use ansi_to_tui::IntoText as _;
 use ratatui::buffer::Buffer;
 use ratatui::layout::Rect;
 use ratatui::prelude::Position;
-use ratatui::style::{Color, Modifier, Style};
+use ratatui::style::Styled;
+use ratatui::style::{Modifier, Style};
 use ratatui::symbols::line;
+use ratatui::text::Text;
 use ratatui::text::{Line, Span};
 use ratatui::widgets::Clear;
 use ratatui::widgets::Widget;
@@ -51,7 +55,7 @@ impl ColumnWidthOverrides {
 
     /// Returns the list of origin column indices that have width overrides
     pub fn overriden_indices(&self) -> Vec<usize> {
-        self.overrides.keys().cloned().collect()
+        self.overrides.keys().copied().collect()
     }
 
     pub fn reset(&mut self) {
@@ -92,7 +96,7 @@ impl<'a> CsvTable<'a> {
 
         let overriden_indices = overrides.overriden_indices();
 
-        for row in self.rows.iter() {
+        for row in self.rows {
             for (i, value) in row.fields.iter().enumerate() {
                 if i >= column_widths.len() {
                     continue;
@@ -239,7 +243,7 @@ impl<'a> CsvTable<'a> {
         let mut y = area.y;
         for (i, row) in rows.iter().enumerate() {
             let row_num_formatted = row.record_num.to_string();
-            let mut style = Style::default().fg(Color::Rgb(64, 64, 64));
+            let mut style = Style::default().fg(state.theme.row_number);
             if let Some(selection) = &state.selection {
                 if selection.row.is_selected(i) {
                     style = style
@@ -262,10 +266,10 @@ impl<'a> CsvTable<'a> {
         });
     }
 
-    fn render_header_borders(&self, buf: &mut Buffer, area: Rect) -> (u16, u16) {
+    fn render_header_borders(&self, buf: &mut Buffer, area: Rect, theme: &Theme) -> (u16, u16) {
         let block = Block::default()
             .borders(Borders::TOP | Borders::BOTTOM)
-            .border_style(Style::default().fg(Color::Rgb(64, 64, 64)));
+            .border_style(Style::default().fg(theme.border));
         let height = 3;
         let area = Rect::new(0, 0, area.width, height);
         block.render(area, buf);
@@ -288,10 +292,12 @@ impl<'a> CsvTable<'a> {
             return;
         }
 
+        let border_style = Style::default().fg(state.theme.border);
+
         // Line number separator
         let line_number_block = Block::default()
             .borders(Borders::RIGHT)
-            .border_style(Style::default().fg(Color::Rgb(64, 64, 64)));
+            .border_style(border_style);
         let line_number_area = Rect::new(0, y_first_record, section_width, area.height);
         line_number_block.render(line_number_area, buf);
 
@@ -303,7 +309,7 @@ impl<'a> CsvTable<'a> {
         // Status separator at the bottom (rendered here first for the interesection)
         let block = Block::default()
             .borders(Borders::TOP)
-            .border_style(Style::default().fg(Color::Rgb(64, 64, 64)));
+            .border_style(border_style);
         let status_separator_area = Rect::new(0, y_first_record + area.height, area.width, 1);
         block.render(status_separator_area, buf);
 
@@ -323,14 +329,13 @@ impl<'a> CsvTable<'a> {
                 col_ending_pos_x,
                 y_first_record.saturating_sub(1),
             )) {
-                cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
+                cell.set_style(border_style)
                     .set_symbol(line::HORIZONTAL_DOWN);
             }
 
             for y in y_first_record..y_first_record + area.height {
                 if let Some(cell) = buf.cell_mut(Position::new(col_ending_pos_x, y)) {
-                    cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
-                        .set_symbol(line::VERTICAL);
+                    cell.set_style(border_style).set_symbol(line::VERTICAL);
                 }
             }
 
@@ -338,8 +343,7 @@ impl<'a> CsvTable<'a> {
                 col_ending_pos_x,
                 y_first_record + area.height,
             )) {
-                cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
-                    .set_symbol(line::HORIZONTAL_UP);
+                cell.set_style(border_style).set_symbol(line::HORIZONTAL_UP);
             }
         }
 
@@ -355,13 +359,12 @@ impl<'a> CsvTable<'a> {
                     x_freeze_separator,
                     y_first_record.saturating_sub(1),
                 )) {
-                    cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
-                        .set_symbol("╥");
+                    cell.set_style(border_style).set_symbol("╥");
                 }
 
                 for y in y_first_record..y_first_record + area.height {
                     if let Some(cell) = buf.cell_mut(Position::new(x_freeze_separator, y)) {
-                        cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
+                        cell.set_style(border_style)
                             .set_symbol(line::DOUBLE_VERTICAL);
                     }
                 }
@@ -370,8 +373,7 @@ impl<'a> CsvTable<'a> {
                     x_freeze_separator,
                     y_first_record + area.height,
                 )) {
-                    cell.set_style(Style::default().fg(Color::Rgb(64, 64, 64)))
-                        .set_symbol("╨");
+                    cell.set_style(border_style).set_symbol("╨");
                 }
             }
         }
@@ -427,6 +429,10 @@ impl<'a> CsvTable<'a> {
             }
             let effective_width = min(remaining_width, hlen);
             let mut content_style = Style::default();
+            if state.color_columns {
+                content_style = content_style
+                    .fg(state.theme.column_colors[col_index % state.theme.column_colors.len()]);
+            }
             if let RowType::Header = row_type {
                 content_style = content_style.add_modifier(Modifier::BOLD);
                 if let Some(selection) = &state.selection {
@@ -443,8 +449,8 @@ impl<'a> CsvTable<'a> {
             let mut filler_style = Style::default();
             if is_selected {
                 let selected_style = Style::default()
-                    .fg(Color::Rgb(192, 192, 192))
-                    .bg(Color::Rgb(64, 64, 64))
+                    .fg(state.theme.selected_foreground)
+                    .bg(state.theme.selected_background)
                     .add_modifier(Modifier::BOLD);
                 filler_style = filler_style.patch(selected_style);
                 content_style = content_style.patch(selected_style);
@@ -475,7 +481,7 @@ impl<'a> CsvTable<'a> {
                 // TODO: seems like doing a bit too much of heavy lifting of
                 // checking for matches (finder's work)
                 FinderState::FinderActive(active) if should_highlight_cell(active, hname) => {
-                    let mut highlight_style = filler_style.style.fg(Color::Rgb(200, 0, 0));
+                    let mut highlight_style = filler_style.style.fg(state.theme.found);
                     if let Some(found_record) = &active.found_record {
                         match found_record {
                             find::FoundEntry::Row(entry) => {
@@ -483,7 +489,8 @@ impl<'a> CsvTable<'a> {
                                     if row_index == entry.row_index()
                                         && entry.column_index() == col_index
                                     {
-                                        highlight_style = highlight_style.bg(Color::LightYellow);
+                                        highlight_style = highlight_style
+                                            .bg(state.theme.found_selected_background);
                                     }
                                 }
                             }
@@ -491,7 +498,8 @@ impl<'a> CsvTable<'a> {
                                 if matches!(row_type, RowType::Header)
                                     && entry.column_index() == col_index
                                 {
-                                    highlight_style = highlight_style.bg(Color::LightYellow);
+                                    highlight_style =
+                                        highlight_style.bg(state.theme.found_selected_background);
                                 }
                             }
                         }
@@ -669,12 +677,15 @@ impl<'a> CsvTable<'a> {
 
     fn render_status(&self, area: Rect, buf: &mut Buffer, state: &mut CsvTableState) {
         // Content of status line (separator already plotted elsewhere)
-        let style = Style::default().fg(Color::Rgb(128, 128, 128));
+        let style = Style::default().fg(state.theme.status);
+        let mut prompt_text: Text;
         let mut content: String;
         state.cursor_xy = None;
         if let Some(msg) = &state.transient_message {
+            prompt_text = Text::default();
             content = msg.to_owned();
         } else if let BufferState::Enabled(buffer_mode, input) = &state.buffer_content {
+            prompt_text = Text::default();
             let get_prefix = |&input_mode| {
                 let prefix = match input_mode {
                     InputMode::GotoLine => "Go to line",
@@ -700,8 +711,16 @@ impl<'a> CsvTable<'a> {
                 area.bottom().saturating_sub(1),
             ));
         } else {
+            // User provided prompt
+            prompt_text = if let Some(prompt) = &state.prompt {
+                prompt.into_text().unwrap_or(Text::default())
+            } else {
+                Text::default()
+            };
             // Filename
-            if let Some(f) = &state.filename {
+            if state.prompt.is_some() {
+                content = "".to_string();
+            } else if let Some(f) = &state.filename {
                 content = f.to_string();
             } else {
                 content = "stdin".to_string();
@@ -774,8 +793,10 @@ impl<'a> CsvTable<'a> {
                 content += format!(" (debug: {})", state.debug).as_str();
             }
         }
-        let span = Span::styled(content, style);
-        buf.set_span(area.x, area.bottom().saturating_sub(1), &span, area.width);
+        prompt_text = prompt_text.set_style(style);
+        prompt_text.push_span(Span::from(content));
+        let prompt_area = Rect::new(area.x, area.y + 1, area.width, area.height);
+        prompt_text.render(prompt_area, buf);
     }
 
     fn get_view_layout(&self, area: Rect, state: &mut CsvTableState, rows: &[Row]) -> ViewLayout {
@@ -844,7 +865,7 @@ impl StatefulWidget for CsvTable<'_> {
         let layout = self.get_view_layout(area, state, self.rows);
         state.view_layout = Some(layout.clone());
 
-        let (y_header, y_first_record) = self.render_header_borders(buf, area);
+        let (y_header, y_first_record) = self.render_header_borders(buf, area, &state.theme);
 
         // row area: including row numbers and row content
         let rows_area = Rect::new(
@@ -1180,8 +1201,7 @@ impl DebugStats {
                     .reader_stats
                     .pos_table_elapsed
                     .as_ref()
-                    .map(|e| e.as_micros())
-                    .unwrap_or(0),
+                    .map_or(0, |e| e.as_micros()),
                 stats.reader_stats.pos_table_entry,
                 stats.reader_stats.num_seek,
                 stats.reader_stats.num_parsed_record
@@ -1225,6 +1245,9 @@ pub struct CsvTableState {
     pub is_word_wrap: bool,
     pub column_width_overrides: ColumnWidthOverrides,
     pub cursor_xy: Option<(u16, u16)>,
+    pub theme: Theme,
+    pub color_columns: bool,
+    pub prompt: Option<String>,
     pub debug: String,
 }
 
@@ -1234,6 +1257,8 @@ impl CsvTableState {
         total_cols: usize,
         echo_column: &Option<String>,
         ignore_case: bool,
+        color_columns: bool,
+        prompt: Option<String>,
     ) -> Self {
         Self {
             rows_offset: 0,
@@ -1259,6 +1284,9 @@ impl CsvTableState {
             is_word_wrap: false,
             column_width_overrides: ColumnWidthOverrides::new(),
             cursor_xy: None,
+            theme: Theme::default(),
+            color_columns,
+            prompt,
             debug: "".into(),
         }
     }
@@ -1285,7 +1313,7 @@ impl CsvTableState {
     }
 
     pub fn has_more_cols_to_show(&self) -> bool {
-        self.more_cols_to_show.map_or(true, |v| v)
+        self.more_cols_to_show.is_none_or(|v| v)
     }
 
     pub fn set_total_line_number(&mut self, n: usize, is_approx: bool) {
@@ -1307,8 +1335,7 @@ impl CsvTableState {
     pub fn line_number_and_spaces_width(&self) -> u16 {
         self.borders_state
             .as_ref()
-            .map(|bs| bs.x_row_separator)
-            .unwrap_or(0)
+            .map_or(0, |bs| bs.x_row_separator)
             + NUM_SPACES_AFTER_LINE_NUMBER
     }
 
