@@ -1,10 +1,10 @@
+use crate::columns_filter::ColumnsFilter;
 use crate::csv::{CsvLensReader, Row};
 use crate::errors::CsvlensResult;
 use crate::find;
 use crate::input::Control;
 use crate::sort::{SortOrder, Sorter};
 
-use regex::Regex;
 use std::cmp::min;
 use std::sync::Arc;
 use std::time::{Duration, Instant};
@@ -27,67 +27,6 @@ impl RowsFilter {
     }
 }
 
-#[derive(Debug)]
-pub struct ColumnsFilter {
-    pattern: Regex,
-    indices: Vec<usize>,
-    filtered_headers: Vec<String>,
-    num_columns_before_filter: usize,
-    disabled_because_no_match: bool,
-}
-
-impl ColumnsFilter {
-    fn new(pattern: Regex, headers: &[String]) -> Self {
-        let mut indices = vec![];
-        let mut filtered_headers: Vec<String> = vec![];
-        for (i, header) in headers.iter().enumerate() {
-            if pattern.is_match(header) {
-                indices.push(i);
-                filtered_headers.push(header.clone());
-            }
-        }
-        let disabled_because_no_match;
-        if indices.is_empty() {
-            indices = (0..headers.len()).collect();
-            filtered_headers = headers.into();
-            disabled_because_no_match = true;
-        } else {
-            disabled_because_no_match = false;
-        }
-        Self {
-            pattern,
-            indices,
-            filtered_headers,
-            num_columns_before_filter: headers.len(),
-            disabled_because_no_match,
-        }
-    }
-
-    fn filtered_headers(&self) -> &Vec<String> {
-        &self.filtered_headers
-    }
-
-    fn indices(&self) -> &Vec<usize> {
-        &self.indices
-    }
-
-    pub fn pattern(&self) -> Regex {
-        self.pattern.to_owned()
-    }
-
-    pub fn num_filtered(&self) -> usize {
-        self.indices.len()
-    }
-
-    pub fn num_original(&self) -> usize {
-        self.num_columns_before_filter
-    }
-
-    pub fn disabled_because_no_match(&self) -> bool {
-        self.disabled_because_no_match
-    }
-}
-
 #[derive(Clone)]
 pub struct SelectionDimension {
     index: Option<u64>,
@@ -254,6 +193,59 @@ pub struct PerfStats {
     pub reader_stats: crate::csv::GetRowsStats,
 }
 
+#[derive(Debug, Clone, Copy, Default)]
+pub struct ColumnsOffset {
+    /// Number of columns that are frozen on the left side (always visible)
+    pub num_freeze: u64,
+
+    /// Number of columns that are skipped after the frozen columns (not visible)
+    pub num_skip: u64,
+}
+
+impl ColumnsOffset {
+    /// Check if the column is frozen
+    pub fn is_frozen(&self, filtered_columns_index: u64) -> bool {
+        filtered_columns_index < self.num_freeze
+    }
+
+    /// Get the index of the column in the columns-filtered data given the view port column index
+    pub fn get_filtered_column_index(&self, view_port_column_index: u64) -> u64 {
+        if view_port_column_index < self.num_freeze {
+            return view_port_column_index;
+        }
+        view_port_column_index.saturating_add(self.num_skip)
+    }
+
+    pub fn should_filtered_column_index_be_rendered(&self, filtered_column_index: u64) -> bool {
+        if filtered_column_index < self.num_freeze {
+            return true;
+        }
+        filtered_column_index >= self.num_freeze.saturating_add(self.num_skip)
+    }
+
+    pub fn is_filtered_column_index_visible(
+        &self,
+        filtered_column_index: u64,
+        num_cols_rendered: u64,
+    ) -> bool {
+        if filtered_column_index < self.num_freeze {
+            return true;
+        }
+        let rendered_start_index = self.num_freeze.saturating_add(self.num_skip);
+        let num_non_frozen_cols_rendered = num_cols_rendered.saturating_sub(self.num_freeze);
+        let rendered_end_index = rendered_start_index.saturating_add(num_non_frozen_cols_rendered);
+        filtered_column_index >= rendered_start_index && filtered_column_index < rendered_end_index
+    }
+
+    pub fn get_num_skip_to_make_visible(&self, filtered_column_index: u64) -> u64 {
+        if filtered_column_index < self.num_freeze {
+            0
+        } else {
+            filtered_column_index.saturating_sub(self.num_freeze)
+        }
+    }
+}
+
 pub struct RowsView {
     reader: CsvLensReader,
     rows: Vec<Row>,
@@ -261,9 +253,9 @@ pub struct RowsView {
     num_rows: u64,
     num_rows_rendered: u64,
     rows_from: u64,
-    cols_offset: u64,
+    cols_offset: ColumnsOffset,
     filter: Option<RowsFilter>,
-    columns_filter: Option<ColumnsFilter>,
+    columns_filter: Option<Arc<ColumnsFilter>>,
     sorter: Option<Arc<Sorter>>,
     sort_order: SortOrder,
     pub selection: Selection,
@@ -282,7 +274,7 @@ impl RowsView {
             num_rows,
             num_rows_rendered: num_rows,
             rows_from,
-            cols_offset: 0,
+            cols_offset: ColumnsOffset::default(),
             filter: None,
             columns_filter: None,
             sorter: None,
@@ -312,6 +304,13 @@ impl RowsView {
             .unwrap_or_default()
     }
 
+    pub fn get_column_name_from_local_index(&self, column_index: usize) -> String {
+        self.headers()
+            .get(column_index)
+            .map(|header| header.name.clone())
+            .unwrap_or_default()
+    }
+
     pub fn get_cell_value(&self, column_name: &str) -> Option<String> {
         if let (Some(column_index), Some(row_index)) = (
             self.headers()
@@ -340,7 +339,7 @@ impl RowsView {
                 .get(row_index as usize)
                 .and_then(|row| {
                     row.fields
-                        .get(column_index.saturating_add(self.cols_offset()) as usize)
+                        .get(self.cols_offset.get_filtered_column_index(column_index) as usize)
                 })
                 .cloned();
         }
@@ -406,22 +405,22 @@ impl RowsView {
         self.do_get_rows()
     }
 
-    pub fn columns_filter(&self) -> Option<&ColumnsFilter> {
+    pub fn columns_filter(&self) -> Option<&Arc<ColumnsFilter>> {
         self.columns_filter.as_ref()
     }
 
-    pub fn set_columns_filter(&mut self, target: Regex) -> CsvlensResult<()> {
-        let _columns_filter = ColumnsFilter::new(target, &self.reader.headers);
-        self.headers = _columns_filter
+    pub fn set_columns_filter(&mut self, columns_filter: &Arc<ColumnsFilter>) -> CsvlensResult<()> {
+        let columns_filter = columns_filter.clone();
+        self.headers = columns_filter
             .indices()
             .iter()
-            .zip(_columns_filter.filtered_headers())
+            .zip(columns_filter.filtered_headers())
             .map(|(i, h)| Header {
                 name: h.clone(),
                 origin_index: *i,
             })
             .collect();
-        self.columns_filter = Some(_columns_filter);
+        self.columns_filter = Some(columns_filter);
         self.do_get_rows()
     }
 
@@ -489,12 +488,23 @@ impl RowsView {
 
     /// Offset of the first column to show. All columns are still read into Row
     /// (per ColumnsFilter if any).
-    pub fn cols_offset(&self) -> u64 {
+    pub fn cols_offset(&self) -> ColumnsOffset {
         self.cols_offset
     }
 
-    pub fn set_cols_offset(&mut self, cols_offset: u64) {
-        self.cols_offset = min(cols_offset, self.headers().len() as u64);
+    pub fn set_cols_offset_num_skip(&mut self, cols_offset: u64) {
+        self.cols_offset.num_skip = min(cols_offset, self.max_cols_offset_num_skip());
+    }
+
+    pub fn set_cols_offset_num_freeze(&mut self, num_freeze: u64) {
+        self.cols_offset.num_freeze =
+            min(num_freeze, self.headers().len().saturating_sub(1) as u64);
+    }
+
+    pub fn max_cols_offset_num_skip(&self) -> u64 {
+        (self.headers().len() as u64)
+            .saturating_sub(self.cols_offset.num_freeze)
+            .saturating_sub(1)
     }
 
     pub fn selected_offset(&self) -> Option<u64> {
